@@ -85,6 +85,9 @@ class TxInput:
     def __str__(self) -> str:  # pragma: no cover
         return f'<TxInput outpoint={self.txid}:{self.vout} satoshi={self.satoshi} locking_script={self.locking_script}>'
 
+    def __repr__(self) -> str:  # pragma: no cover
+        return self.__str__()
+
     @classmethod
     def from_hex(cls, stream: Union[str, bytes, TransactionBytesIO]) -> Optional['TxInput']:
         with suppress(Exception):
@@ -127,6 +130,9 @@ class TxOutput:
     def __str__(self) -> str:  # pragma: no cover
         return f'<TxOutput satoshi={self.satoshi} locking_script={self.locking_script.hex()}>'
 
+    def __repr__(self) -> str:  # pragma: no cover
+        return self.__str__()
+
     @classmethod
     def from_hex(cls, stream: Union[str, bytes, TransactionBytesIO]) -> Optional['TxOutput']:
         with suppress(Exception):
@@ -358,8 +364,6 @@ class Transaction:
                 # unlocking script already set
                 estimated_length += len(tx_input.serialize())
             else:
-                if not tx_input.private_keys:
-                    raise ValueError(f"can't estimate byte length for {tx_input} without private keys")
                 estimated_length += 41 + tx_input.script_type.estimated_unlocking_byte_length(private_keys=tx_input.private_keys, **{**self.kwargs, **kwargs})
         for tx_output in self.tx_outputs:
             estimated_length += 8 + len(tx_output.locking_script.byte_length_varint()) + tx_output.locking_script.byte_length()
@@ -382,10 +386,8 @@ class Transaction:
         if fee_overpaid >= P2PKH_DUST_LIMIT:
             change_output: Optional[TxOutput] = None
             if not change_address:
-                for tx_input in self.tx_inputs:  # pragma: no cover
-                    if tx_input.script_type == P2pkhScriptType():
-                        change_output = TxOutput(out=tx_input.locking_script, satoshi=fee_overpaid, script_type=P2pkhScriptType())
-                        break
+                if self.tx_inputs:  # pragma: no cover
+                    change_output = TxOutput(out=self.tx_inputs[0].locking_script, satoshi=fee_overpaid)
             else:
                 change_output = TxOutput(out=change_address, satoshi=fee_overpaid)
             assert change_output, "can't parse any address from transaction inputs"
